@@ -56,12 +56,12 @@ $attributes_map = array(
     'employeenumber' => array( 'attribute' => 'employeenumber', 'faclass' => 'hashtag', 'type' => 'text' ),
     'employeetype' => array( 'attribute' => 'employeetype', 'faclass' => 'id-badge', 'type' => 'text' ),
     'fax' => array( 'attribute' => 'facsimiletelephonenumber', 'faclass' => 'fax', 'type' => 'tel', 'dtorder' => 'disable' ),
-    'firstname' => array( 'attribute' => 'givenname', 'faclass' => 'user-o', 'type' => 'text' , 'dtorder' => 'disable' ),
+    'firstname' => array( 'attribute' => 'givenname', 'mandatory' => array("all"), 'faclass' => 'user-o', 'type' => 'text' , 'dtorder' => 'disable' ),
     'fullname' => array( 'attribute' => 'cn', 'faclass' => 'user-circle', 'type' => 'text' ),
     'l' => array( 'attribute' => 'l', 'faclass' => 'globe', 'type' => 'text' ),
-    'lastname' => array( 'attribute' => 'sn', 'faclass' => 'user-o', 'type' => 'text' ),
-    'mail' => array( 'attribute' => 'mail', 'faclass' => 'envelope-o', 'type' => 'mailto' ),
-    'mailquota' => array( 'attribute' => 'gosamailquota', 'faclass' => 'pie-chart', 'type' => 'bytes', 'dtorder' => 'disable' ),
+    'lastname' => array( 'attribute' => 'sn', 'mandatory' => array("all"), 'faclass' => 'user-o', 'type' => 'text' ),
+    'mail' => array( 'attribute' => 'mail', 'faclass' => 'envelope-o', 'type' => 'mailto', 'multivalued' => true ),
+    'mailquota' => array( 'attribute' => 'gosamailquota', 'faclass' => 'pie-chart', 'type' => 'bytes' ),
     'manager' => array( 'attribute' => 'manager', 'faclass' => 'user-circle-o', 'type' => 'dn_link', 'dtorder' => 'disable' ),
     'mobile' => array( 'attribute' => 'mobile', 'faclass' => 'mobile', 'type' => 'tel', 'dtorder' => 'disable' ),
     'modified' => array( 'attribute' => 'modifytimestamp', 'faclass' => 'clock-o', 'type' => 'date' ),
@@ -80,25 +80,41 @@ $attributes_map = array(
 # Directory specific attributes
 $openldap_attributes_map = array(
     'authtimestamp' => array( 'attribute' => 'authtimestamp', 'faclass' => 'lock', 'type' => 'date' ),
-    'identifier' => array( 'attribute' => 'uid', 'faclass' => 'user-o', 'type' => 'text' ),
+    'identifier' => array( 'attribute' => 'uid', 'mandatory' => array("all"), 'faclass' => 'user-o', 'type' => 'text' ),
     'pwdaccountlockedtime' => array( 'attribute' => 'pwdaccountlockedtime', 'faclass' => 'lock', 'type' => 'date' ),
     'pwdchangedtime' => array( 'attribute' => 'pwdchangedtime', 'faclass' => 'lock', 'type' => 'date', 'sort' => 'descending' ),
     'pwdfailuretime' => array( 'attribute' => 'pwdfailuretime', 'faclass' => 'lock', 'type' => 'date', 'sort' => 'descending' ),
+    'pwdgraceusetime' => array( 'attribute' => 'pwdgraceusetime', 'faclass' => 'lock', 'type' => 'date' ),
+    'pwdhistory' => array( 'attribute' => 'pwdhistory', 'faclass' => 'lock', 'type' => 'pwdhistory' ),
     'pwdlastsuccess' => array( 'attribute' => 'pwdlastsuccess', 'faclass' => 'lock', 'type' => 'date' ),
     'pwdpolicysubentry' => array( 'attribute' => 'pwdpolicysubentry', 'faclass' => 'lock', 'type' => 'ppolicy_dn' ),
-    'pwdgraceusetime' => array( 'attribute' => 'pwdgraceusetime', 'faclass' => 'lock', 'type' => 'date' ),
     'starttime' => array( 'attribute' => 'pwdstarttime', 'faclass' => 'lock', 'type' => 'date' ),
     'endtime' => array( 'attribute' => 'pwdendtime', 'faclass' => 'lock', 'type' => 'date' ),
 );
 $activedirectory_attributes_map = array(
     'authtimestamp' => array( 'attribute' => 'lastlogon', 'faclass' => 'lock', 'type' => 'ad_date' ),
-    'identifier' => array( 'attribute' => 'samaccountname', 'faclass' => 'user-o', 'type' => 'text' ),
+    'identifier' => array( 'attribute' => 'samaccountname', 'mandatory' => array("all"), 'faclass' => 'user-o', 'type' => 'text' ),
     'pwdaccountlockedtime' => array( 'attribute' => 'lockouttime', 'faclass' => 'lock', 'type' => 'ad_date' ),
     'pwdchangedtime' => array( 'attribute' => 'pwdlastset', 'faclass' => 'lock', 'type' => 'ad_date' ),
     'pwdfailuretime' => array( 'attribute' => 'badpasswordtime', 'faclass' => 'lock', 'type' => 'ad_date' ),
     'endtime' => array( 'attribute' => 'accountexpires', 'faclass' => 'lock', 'type' => 'ad_date' ),
 );
 
+# List of values
+// Static lists: key/values in configuration file
+$attributes_static_list = array();
+// $attributes_static_list['title'] = array( 'Mr' => 'Mister', 'Mrs' => 'Misses' );
+
+// LDAP lists: key/values returned by an LDAP search
+$attributes_list = array();
+// $attributes_list['organizationalunit'] = array('base'=>'ou=services,dc=example,dc=com', 'filter'=>'(objectClass=organizationalUnit)', 'key'=>'description', 'value'=>'ou');
+
+# DN Link
+$dn_link_label_attributes = array("cn");
+#$dn_link_search_display_macro = "%fullname% (%mail%)";
+$dn_link_search_min_chars = 3;
+$dn_link_search_size_limit = 10;
+
 # Search
 $search_attributes = array('uid', 'cn', 'mail');
 $search_use_substring_match = true;
@@ -118,7 +134,7 @@ $datatables_auto_print = true;
 $display_items = array('identifier', 'firstname', 'lastname', 'title', 'businesscategory', 'employeenumber', 'employeetype', 'mail', 'mailquota', 'phone', 'mobile', 'fax', 'postaladdress', 'street', 'postalcode', 'l', 'state', 'organizationalunit', 'organization', 'manager', 'secretary' );
 $display_title = "fullname";
 $display_show_undefined = false;
-$display_password_items = array('pwdchangedtime', 'pwdfailuretime','pwdpolicysubentry', 'authtimestamp', 'pwdgraceusetime', 'starttime', 'endtime', 'created', 'modified');
+$display_password_items = array('pwdchangedtime', 'pwdfailuretime', 'pwdhistory', 'pwdpolicysubentry', 'authtimestamp', 'pwdgraceusetime', 'starttime', 'endtime', 'created', 'modified');
 $display_password_expiration_date = true;
 
 # Features
@@ -167,6 +183,33 @@ $use_updatestarttime = true;
 $use_updateendtime = true;
 $use_searchinvalid = true;
 
+$use_searchall = true;
+
+$samba_mode = false;
+$samba_options=[];
+# Set password min/max age in Samba attributes
+#$samba_options['min_age'] = 5;
+#$samba_options['max_age'] = 45;
+#$samba_options['expire_days'] = 90;
+
+## Edition
+
+$use_update = true;
+$update_items = array('firstname', 'lastname', 'title', 'businesscategory', 'employeenumber', 'employeetype', 'mail', 'mailquota', 'phone', 'mobile', 'fax', 'postaladdress', 'street', 'postalcode', 'l', 'state', 'organizationalunit', 'organization', 'manager', 'secretary');
+$update_items_macros = array('fullname' => '%firstname% %lastname%');
+
+$use_rename = true;
+$rename_items = array('identifier');
+
+$use_create = true;
+$create_items = array('identifier', 'firstname', 'lastname', 'mail');
+$create_objectclass = array('top', 'person', 'organizationalPerson', 'inetOrgPerson');
+$create_dn_items = array('identifier');
+$create_base = $ldap_user_base;
+$create_items_macros = array('fullname' => '%firstname% %lastname%');
+
+$use_delete = true;
+
 # Local password policy
 # This is applied before directory password policy
 # Minimal length
@@ -213,6 +256,9 @@ $pwd_show_policy = "never";
 # below - the form
 $pwd_show_policy_pos = "above";
 
+# disallow use of the only special character as defined in `$pwd_special_chars` at the beginning and end
+$pwd_no_special_at_ends = false;
+
 ## Mail
 # LDAP mail attribute
 $mail_attributes = array( "mail", "gosaMailAlternateAddress", "proxyAddresses" );
@@ -253,16 +299,21 @@ $mail_priority = 3;
 $lang = "en";
 $allowed_lang = array();
 $date_specifiers = "%Y-%m-%d %H:%M:%S (%Z)";
+$js_date_specifiers = "YYYY-MM-DD HH:mm:ss (Z)";
 $date_timezone = "UTC";
 
 # Graphics
 $logo = "images/ltb-logo.png";
 $background_image = "images/unsplash-space.jpeg";
+$favicon = "images/favicon.ico";
 $custom_css = "";
 $display_footer = true;
 #$logout_link = "http://auth.example.com/logout";
 $fake_password_inputs = false;
 
+# Customized template directory: to override some templates files
+$custom_tpl_dir = "";
+
 # Audit
 #$audit_log_file = "/var/log/service-desk/audit.log";
 $use_showauditlog = false;
@@ -276,97 +327,79 @@ $audit_log_truncate_value_after = 40;
 # Debug mode
 $debug = false;
 
-### Prehooks
+$debug_level = E_ALL & ~E_NOTICE & ~E_STRICT & ~E_DEPRECATED & ~E_WARNING;
 
-# Launch a prehook script before an action.
-# Script should return with 0, else action will be aborted, unless error is ignored
+# Hooks
 
-# LDAP attribute used as login in posthook script
-$prehook_login = "uid";
-
-## Password reset
-
-#$prehook = "/usr/share/service-desk/prehook.sh";
-
-# Display prehook error
-#$display_prehook_error = true;
-# Encode passwords sent to prehook script as base64. This will prevent alteration of the passwords if set to true.
-# To read the actual password in the prehook script, use a base64_decode function/tool
-#$prehook_password_encodebase64 = false;
-# Ignore prehook error. This will allow to change password even if prehook script fails.
-#$ignore_prehook_error = true;
-
-## Lock
-
-#$prehook_lock = "/usr/share/service-desk/prehook_lock.sh";
-#$display_prehook_lock_error = true;
-#$ignore_prehook_lock_error = true;
-
-## Unlock
-
-#$prehook_unlock = "/usr/share/service-desk/prehook_unlock.sh";
-#$display_prehook_unlock_error = true;
-#$ignore_prehook_unlock_error = true;
-
-## Enable
-
-#$prehook_enable = "/usr/share/service-desk/prehook_enable.sh";
-#$display_prehook_enable_error = true;
-#$ignore_prehook_enable_error = true;
-
-## Disable
-
-#$prehook_disable = "/usr/share/service-desk/prehook_disable.sh";
-#$display_prehook_disable_error = true;
-#$ignore_prehook_disable_error = true;
-
-## Update validity
-
-#$prehook_updatevalidity = "/usr/share/service-desk/prehook_updatevalidity.sh";
-#$display_prehook_updatevalidity_error = true;
-#$ignore_prehook_updatevalidity_error = true;
-
-### Posthooks
-
-# The posthook is only launched if the action was successful
-
-# LDAP attribute used as login in posthook script
-$posthook_login = "uid";
-
-## Password reset
-
-#$posthook = "/usr/share/service-desk/posthook.sh";
-
-# Display posthook error
-#$display_posthook_error = true;
-# Encode passwords sent to posthook script as base64. This will prevent alteration of the passwords if set to true.
-# To read the actual password in the posthook script, use a base64_decode function/tool
-#$posthook_password_encodebase64 = false;
-
-## Lock
-
-#$posthook_lock = "/usr/share/service-desk/posthook_lock.sh";
-#$display_posthook_lock_error = true;
-
-## Unlock
-
-#$posthook_unlock = "/usr/share/service-desk/posthook_unlock.sh";
-#$display_posthook_unlock_error = true;
-
-## Enable
-
-#$posthook_enable = "/usr/share/service-desk/posthook_enable.sh";
-#$display_posthook_enable_error = true;
-
-## Disable
-
-#$posthook_disable = "/usr/share/service-desk/posthook_disable.sh";
-#$display_posthook_disable_error = true;
-
-## Update validity
-
-#$posthook_updatevalidity = "/usr/share/service-desk/posthook_updatevalidity.sh";
-#$display_posthook_updatevalidity_error = true;
+# hook_login_attribute: LDAP attribute used as identifier to send to the script or function
+$hook_login_attribute = "uid";
+#$hook_config = array(
+#    "passwordReset" => array(
+#        "before" => array(
+#            "externalScript" => "/usr/share/service-desk/hook.sh",
+#            "function" => "hookFunction",
+#            "displayError" => false,
+#            "encodebase64" => false,
+#            "ignoreError" => false
+#        ),
+#        "after" => array(
+#            "externalScript" => "/usr/share/service-desk/hook.sh",
+#            "function" => "hookFunction",
+#            "displayError" => false,
+#            "encodebase64" => false
+#        )
+#    ),
+#    "passwordCheck" => array(),
+#    "passwordLock" => array(),
+#    "passwordUnlock" => array(),
+#    "accountEnable" => array(),
+#    "accountDisable" => array(),
+#    "updateValidityDates" => array(),
+#    "createAccount" => array(),
+#    "updateAccount" => array(),
+#    "deleteAccount" => array(),
+#    "renameAccount" => array()
+#);
+#
+# Hook entrypoints:
+# passwordReset, passwordLock,...: entrypoints triggering hooks
+#  * passwordCheck:       input: login, new password
+#  * passwordReset:       input: login, new password
+#  * passwordLock:        input: login
+#  * passwordUnlock:      input: login
+#  * accountEnable:       input: login
+#  * accountDisable:      input: login
+#  * updateValidityDates: input: login, start date, end date
+#  * createAccount:       input: dn, ldap entry
+#  * updateAccount:       input: dn, ldap entry
+#  * deleteAccount:       input: login
+#  * renameAccount:       input: login, dn, new_rdn, parent)
+#
+# Hook outputs:
+# * general external script output: first line: error message
+# * general external script return code: 0 is a success, any other value means an error
+# * general function return values: return code, error message
+# * for createAccount:
+#   * for step=before external script, the expected output is: first line: error message, second line: dn, all other lines: ldap entry in json format
+#   * for step=before function, the expected returned values are: return code, error message, dn, ldap entry
+# * for updateAccount:
+#   * for step=before external script, the expected output is: first line: error message, all other lines: ldap entry in json format
+#   * for step=before function, the expected returned values are: return code, error message, ldap entry
+# * for renameAccount:
+#   * for step=before external script, the expected output is: first line: error message, second line: dn, all other lines: JSON document with 'new_rdn' and 'parent' keys
+#   * for step=before function, the expected returned values are: return code, error message, dn, array with 'new_rdn' and 'parent' keys
+#
+# Hook steps:
+# * before: before the entrypoint
+# * after: after the entrypoint
+#
+# Hook parameters:
+# * externalScript: path of the script that is called. "before" script or function should return 0, else action will be aborted, unless error is ignored
+# * function: the hook can also be a function. Write your own file.php in hooks/ directory
+# * displayError: display an error if the script or function returns an error
+# * ignoreError: only for before hooks, ignore error returned by the script or function
+# * encodebase64: passwordReset and passwordCheck entrypoints only, encode the password in base64 before sending it
+$hook_config = array();
 
 # The name of an HTTP Header that may hold a reference to an extra config file to include.
 #$header_name_extra_config="SSP-Extra-Config";
